/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.admin;

import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.basisapi.exception.PetalsException;

/**
 * This class is a manager of JMX server.
 * <p>
 * It is used to get a connection to a
 * JMX Server on a distant container.
 * </p>
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 */
public interface PetalsAdminServiceMBean {

    /**
     * <p>
     * Retrieve all the properties of the server local configuration (ie. the
     * content of '<code>server.properties</code>' or equivalent, including:
     * <ul>
     * <li>the configured properties and un.
     * </p>
     * <p>
     * Unset properties are returned as set to an empty {@link String}.
     * </p>
     * 
     * @return The properties of the server local configuration.
     */
    public Properties retrieveServerProperties();
    
    /**
     * Retrieve the collection of containers, sub-domains and domain
     * configurations.
     * 
     * @param securityPassPhrase
     *            Passphrase needed to return sensible information. If
     *            <code>null</code>, empty or not matching the one of the
     *            container on which the connection is established, no sensible
     *            information will be returned.
     * @return a Set of containers, sub-domains and domain configurations in
     *         key/value form
     * @throws PetalsException
     */
    public Set<Map<String, String>> retrieveTopology(final String securityPassPhrase)
            throws PetalsException;
    
    /**
     * Ping if the server is ready to process messages in the domain
     */
    public boolean ping();

    /**
     * Unload the container from the Petals global environment. All
     * ServiceAssemblies, Components and Share Libraries are stopped, shutdown,
     * and uninstalled. All registered Endpoints are deactivated. Then the
     * container is removed from the domain.
     * 
     * @throws Exception
     */
    public void shutdownContainer() throws Exception;

    /**
     * Stop the container. The container is still visible in the Petals global
     * environment, but marked as 'stopped'. The registered Endpoints are still
     * visible,too.
     * 
     * @throws Exception
     */
    public void stopContainer() throws Exception;
}
