/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.communication;

import javax.management.MBeanServer;

import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.jmx.api.api.JMXClient;

/**
 * This class is a manager of JMX server.
 * <p>
 * It is used to get a connection of a remote or the local JMX server.
 * </p>
 * 
 * @author ddesjardins - EBM WebSourcing
 */
public interface JMXService {

    public static final String COMPONENT_LOGGER_NAME = "Petals.Communication.JMXService";

    /**
     * Create a connection to the remote container JMX server.
     * 
     * @param remoteContainerName
     *            The remote container name
     * 
     * @return DistributedJMXServer
     * @throws PetalsException
     */
    JMXClient getJMXClient(final String remoteContainerName) throws PetalsException;

    /**
     * Get the local jmx server
     * 
     * @return the local jmx server
     */
    public MBeanServer getLocalJMXServer();

}
