/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.io.Serializable;

/**
 * This class holds the global topology information.
 * <p>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * </p>
 * 
 * @author Roland NAUDIN - EBM WebSourcing
 * @since Petals 2.0
 */
public class DomainConfiguration implements Serializable {

    /**
     * 
     */
    private static final long serialVersionUID = -4951508542017620408L;

    /**
     * Domain mode enumeration.
     */
    public static enum DomainMode {
        /**
         * Static distributed mode
         */
        STATIC,
        /**
         * Dynamic distributed mode where nodes can be attached or detached at
         * runtime
         */
        DYNAMIC,
    };

    /**
     * Domain name.
     */
    private String name;

    /**
     * Domain description
     */
    private String description;

    /**
     * The domain mode.
     */
    private DomainMode mode;

    /**
     * The JNDI configuration.
     */
    private JndiConfiguration jndiConfiguration;

    public String getName() {
        return this.name;
    }

    public String getDescription() {
        return this.description;
    }

    public DomainMode getMode() {
        return this.mode;
    }

    public JndiConfiguration getJndiConfiguration() {
        return this.jndiConfiguration;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setMode(DomainMode mode) {
        this.mode = mode;
    }

    public void setJndiConfiguration(JndiConfiguration jndiConfiguration) {
        this.jndiConfiguration = jndiConfiguration;
    }
    
    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Domain " + this.name + "\n\t\t\t  Configuration :");
        toStringBuilder.append("\n\t\t\t  - Mode:" + this.mode);
        toStringBuilder.append("\n\t\t\t  - JNDI configuration :" + this.jndiConfiguration);

        return toStringBuilder.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((this.name == null) ? 0 : this.name.hashCode());
        result = prime * result + ((this.description == null) ? 0 : this.description.hashCode());
        result = prime * result + ((this.jndiConfiguration == null) ? 0 : this.jndiConfiguration.hashCode());
        result = prime * result + ((this.mode == null) ? 0 : this.mode.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }

        if (obj instanceof DomainConfiguration) {
            final DomainConfiguration otherDomainConfiguration = (DomainConfiguration) obj;

            // mandatory fields
            if (!(this.name.equals(otherDomainConfiguration.name) && this.mode
                    .equals(otherDomainConfiguration.mode))) {
                return false;
            }

            // optional fields
            if ((this.description == null) && (otherDomainConfiguration.description != null)) {
                return false;
            } else if (!this.description.equals(otherDomainConfiguration.description)) {
                return false;
            }

        } else {
            return false;
        }

        return true;
    }
}
