/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.net.URI;

/**
 * This class holds the global topology information.
 * @author Charles Casadei - EBM WebSourcing
 * @since 3.0
 */
public class JndiConfiguration {

    private static final long serialVersionUID = -4815105115781154403L;

    /**
     * JNDI Factory
     */
    private String jndiFactory;

    /**
     * JNDI URL
     */
    private URI jndiProviderUrl;

    /***
     * JNDI security Principal name
     */
    private String jndiSecurityPrincipal;

    /**
     * JNDI security Credentials
     */
    private String jndiSecurityCredentials;

    /**
     * Connection Pool size of the JNDI server
     */
    private int jndiPoolSize;

    /**
     * Batch size of the JNDI server
     */
    private int jndiBatchSize;

    public String getJndiFactory() {
        return this.jndiFactory;
    }

    public URI getJndiProviderUrl() {
        return this.jndiProviderUrl;
    }

    public String getJndiSecurityPrincipal() {
        return this.jndiSecurityPrincipal;
    }

    public String getJndiSecurityCredentials() {
        return this.jndiSecurityCredentials;
    }

    public int getJndiPoolSize() {
        return this.jndiPoolSize;
    }

    public int getJndiBatchSize() {
        return this.jndiBatchSize;
    }
    
    public void setJndiFactory(String jndiFactory) {
        this.jndiFactory = jndiFactory;
    }

    public void setJndiProviderUrl(URI jndiProviderUrl) {
        this.jndiProviderUrl = jndiProviderUrl;
    }

    public void setJndiSecurityPrincipal(String jndiSecurityPrincipal) {
        this.jndiSecurityPrincipal = jndiSecurityPrincipal;
    }

    public void setJndiSecurityCredentials(String jndiSecurityCredentials) {
        this.jndiSecurityCredentials = jndiSecurityCredentials;
    }

    public void setJndiPoolSize(int poolSize) {
        this.jndiPoolSize = poolSize;
    }

    public void setJndiBatchSize(int batchSize) {
        this.jndiBatchSize = batchSize;
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();

        if (this.jndiFactory == null) {
            toStringBuilder.append("\n\t\t\t\t  - JNDIFactory: None");
        } else {
            toStringBuilder.append("\n\t\t\t\t  - JNDIFactory:" + this.jndiFactory);
        }

        if (this.jndiProviderUrl != null) {
            toStringBuilder.append("\n\t\t\t\t  - JNDIProviderUrl:" + this.jndiProviderUrl);
        }

        if (this.jndiSecurityPrincipal != null) {
            toStringBuilder.append("\n\t\t\t\t  - JNDISecurityPrincipal:"
                    + this.jndiSecurityPrincipal);
        }

        if (this.jndiSecurityCredentials != null) {
            toStringBuilder.append("\n\t\t\t\t  - JNDISecurityCredentials:"
                    + this.jndiSecurityCredentials);
        }

        if (this.jndiPoolSize != 0) {
            toStringBuilder.append("\n\t\t\t\t  - JNDIPool size:" + this.jndiPoolSize);
        }

        if (this.jndiBatchSize != 0) {
            toStringBuilder.append("\n\t\t\t\t  - JNDIbatch size:" + this.jndiBatchSize);
        }

        return toStringBuilder.toString();
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;

        result = prime * result + ((this.jndiFactory == null) ? 0 : this.jndiFactory.hashCode());
        result = prime * result
                + ((this.jndiProviderUrl == null) ? 0 : this.jndiProviderUrl.hashCode());
        result = prime
                * result
                + ((this.jndiSecurityPrincipal == null) ? 0 : this.jndiSecurityPrincipal.hashCode());
        result = prime
                * result
                + ((this.jndiSecurityCredentials == null) ? 0 : this.jndiSecurityCredentials.hashCode());
        result = prime * result + this.jndiPoolSize + this.jndiBatchSize;

        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }

        if (obj instanceof JndiConfiguration) {
            final JndiConfiguration otherJndiConfiguration = (JndiConfiguration) obj;

            if ((this.jndiFactory == null) && (otherJndiConfiguration.jndiFactory != null)) {
                return false;
            } else if (!(this.jndiFactory.equals(otherJndiConfiguration.jndiFactory) && (this.jndiProviderUrl
                    .equals(otherJndiConfiguration.jndiProviderUrl) && this.jndiSecurityPrincipal
                    .equals(otherJndiConfiguration.jndiSecurityPrincipal)))) {
                return false;
            }
        } else {
            return false;
        }

        return true;
    }
}
