/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.configuration;

import java.io.Serializable;

import org.ow2.petals.topology.generated.SubdomainMode;

/**
 * This class holds the sub-domain topology information.
 * @author Christophe DENEUX - Capgemini Sud
 * @since Petals 2.2
 */
public class SubDomainConfiguration implements Serializable {

	  /**
     * Unique Serial UID
     */
    private static final long serialVersionUID = 6441320022067994394L;

    /**
     * Sub-domain name.
     */
    private String name;

    /**
     * Sub-domain description
     */
    private String description;

    /**
     * Subdomain mode.
     */
    private SubdomainMode mode;

    /**
     * Is the sub-domain network time-synchronized ? Default value is
     * <code>false</code>
     */
    private boolean networkTimeSynchronized;

    public String getName() {
        return this.name;
    }

    public String getDescription() {
        return this.description;
    }

    public SubdomainMode getMode() {
        return this.mode;
    }

    public boolean isNetworkTimeSynchronized() {
        return this.networkTimeSynchronized;
    }

    @Override
    public String toString() {
        final StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Sub-domain ").append(this.name);

        if (this.description != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.description);
        }
        toStringBuilder.append("\n\t\t\tConfiguration :\n\t\t\t  - Time-synchronized:"
                + String.valueOf(this.networkTimeSynchronized));

        return toStringBuilder.toString();
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }

        if (obj instanceof SubDomainConfiguration) {
            SubDomainConfiguration otherSubdomainConfiguration = (SubDomainConfiguration) obj;

            // mandatory fields
            if (!(otherSubdomainConfiguration.name.equals(this.name) && otherSubdomainConfiguration.networkTimeSynchronized == this.networkTimeSynchronized)) {
                return false;
            }

            // optional field
            if (this.description == null && otherSubdomainConfiguration.description != null) {
                return false;
            } else if (!this.description.equals(otherSubdomainConfiguration.description)) {
                return false;
            }
        } else {
            return false;
        }
        return true;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public void setMode(SubdomainMode mode) {
        this.mode = mode;
    }

    public void setNetworkTimeSynchronized(final boolean networkTimeSynchronized) {
        this.networkTimeSynchronized = networkTimeSynchronized;
    }
}
