/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container;

import java.io.IOException;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.management.InstallerMBean;
import javax.naming.NamingException;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.ow2.petals.basisapi.exception.PetalsException;
import org.ow2.petals.microkernel.api.jbi.management.Context;

/**
 * @author Roland Naudin - EBM WebSourcing
 * @author Christophe DENEUX - EBM Websourcing
 */
public interface Installer extends InstallerMBean, ComponentContextCommunication {

    public static final String COMPONENT_LOGGER_NAME = "Petals.Container.Lifecycle.Installer";

    public static final String INSTALLED = "Installed";

    public static final String UNINSTALLED = "Uninstalled";

    /**
     * init the installer.
     * 
     * @param ctxt
     *            the context (installation context)
     * @throws PetalsException
     *             : impossible initialization
     * @throws JBIException
     *             : impossible initialization
     */
    void init(Context ctxt) throws IOException, JBIException, NamingException,
            NoSuchInterfaceException;

    /**
     * Initialize the bootstrap of the JBI component. Invoked when loading a new
     * installer of a JBI component.
     * 
     * @param install
     * @throws JBIException
     */
    public void initBootstrap(boolean install) throws JBIException;

    /**
     * shutdown the installer freeing its resources
     * 
     * @throws JBIException
     */
    void shutdown() throws JBIException;

    /**
     * @return The JBI component associated to this installer
     */
    public Component getComponent();

    /**
     * @return The JBI identifier of the component associated to this installer
     */
    public String getComponentName();
}
