/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.extension;


/**
 * A component providing features to install, uninstall or deploy/undeploy
 * artifacts <b>MUST</b> implement this interface.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public interface InstallationExtension {

    /**
     * Callback invoked on installation process to register the installed
     * archive in the component implementing this interface.
     * 
     * @param installedArchiveFilename
     *            The installed archive file name, as <b>absolute</b> file name.
     * @throws InstallationExtensionException
     *             An error occurs processing the installed archive.
     */
    public void onInstallation(final String installedArchiveFilename)
            throws InstallationExtensionException;

    /**
     * Callback invoked on cancelation of the installation process to unregister
     * an installed archive from the component implementing this interface.
     * 
     * @param installedArchiveFilename
     *            The installed archive file name previously registered and that
     *            should be cancelled
     * @throws InstallationExtensionException
     *             An error occurs processing the installed archive.
     */
    public void onInstallationUndo(final String installedArchiveFilename)
            throws InstallationExtensionException;

    /**
     * Callback invoked on uninstallation process to register the uninstalled
     * archive in the component implementing this interface.
     * 
     * @param uninstalledArchiveFilename
     *            The uninstalled archive file name
     * @throws InstallationExtensionException
     *             An error occurs processing the uninstalled archive.
     */
    public void onUninstallation(final String uninstalledArchiveFilename)
            throws InstallationExtensionException;

    /**
     * Callback invoked on cancelation of the uninstallation process to register
     * an installed archive from the component implementing this interface.
     * 
     * @param uninstalledArchiveFilename
     *            The uninstalled archive file name previously unregistered and
     *            that should be cancelled
     * @throws InstallationExtensionException
     *             An error occurs processing the installed archive.
     */
    public void onUninstallationUndo(final String uninstalledArchiveFilename)
            throws InstallationExtensionException;

}
