/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;

import java.util.List;
import java.util.Map;

import javax.jbi.management.AdminServiceMBean;
import javax.jbi.management.ComponentLifeCycleMBean;
import javax.jbi.management.MBeanNames;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.ow2.petals.microkernel.api.container.Installer;
import org.ow2.petals.microkernel.api.container.ServiceAssemblyLifeCycle;
import org.ow2.petals.microkernel.api.container.ServiceUnitLifeCycle;
import org.ow2.petals.microkernel.api.container.SharedLibraryLifeCycle;

/**
 * Internal admin methods.
 * @author Roland Naudin - EBM WebSourcing
 */
public interface AdminService extends AdminServiceMBean {

    public static final String COMPONENT_LOGGER_NAME = "Petals.JBI-Management.AdminService";

    /**
     * Get the PEtALS {@code MbeanNames} instance.
     * 
     * @return the instance
     */
    MBeanNames getMBeanNames();

    /**
     * Get the {@code MBeanServer}.
     * 
     * @return the instance
     */
    MBeanServer getMBeanServer();

    /**
     * Get the {@link Installer} by its component name.
     * 
     * @param name
     * @return the {@link Installer}
     */
    Installer getInstallerByName(String name);

    /**
     * Get the list of {@link Installer}s
     * 
     * @return
     */
    Installer[] getInstallers();

    /**
     * Find the {@link ComponentLifeCycleMBean} of a JBI Component by its unique
     * name.
     * 
     * @param name
     *            the name of the binding component; must be non-null and
     *            non-empty
     * @return the {@link ComponentLifeCycleMBean} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
     */
    ComponentLifeCycleMBean getBindingComponentLifeCycleByName(String name);

    Map<String, ComponentLifeCycleMBean> getBindingComponentsLifeCycles();

    Map<String, ComponentLifeCycleMBean> getServiceEnginesLifeCycles();

    /**
     * Find the {@link ComponentLifeCycleMBean} of a JBI Component by its unique
     * name.
     * 
     * @param name
     *            the name of the engine component; must be non-null and
     *            non-empty
     * @return the {@link ComponentLifeCycleMBean} of the component, or
     *         <code>null</code> if there is no such component with the given
     *         <code>name</code>
     */
    ComponentLifeCycleMBean getEngineComponentLifeCycleByName(String name);

    /**
     * Get the {@link ServiceAssemblyLifeCycle} by its unique name.
     * 
     * @param name
     * @return the {@link ServiceAssemblyLifeCycle}
     */
    ServiceAssemblyLifeCycle getServiceAssemblyByName(String saName);

    /**
     * Get the map of {@link ServiceAssemblyLifeCycle} by names.
     * 
     * @return the map
     */
    Map<String, ServiceAssemblyLifeCycle> getServiceAssemblies();

    /**
     * Get the {@link SharedLibraryLifeCycle} by its unique name.
     * 
     * @param name
     * @return the {@link SharedLibraryLifeCycle}
     */
    SharedLibraryLifeCycle getSharedLibraryByName(String slName);

    /**
     * Get the map of {@link SharedLibraryLifeCycle} by names.
     * 
     * @return the map, not <code>null</code>
     */
    Map<String, SharedLibraryLifeCycle> getSharedLibraries();

    List<ServiceUnitLifeCycle> getServiceUnitsLifeCyclesForComponent(String component);

    ObjectName registerBindingComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException;

    ObjectName registerEngineComponent(ComponentLifeCycleMBean lifeCycle, String name)
            throws ManagementException;

    ObjectName registerInstaller(Installer installerMBean, String name) throws ManagementException;

    void registerServiceAssembly(ServiceAssemblyLifeCycle serviceAssemblyLifeCycle, String name);

    void registerSharedLibrary(SharedLibraryLifeCycle sharedLibraryLifeCycle, String name);

    void unregisterBindingComponent(String name) throws ManagementException;

    void unregisterEngineComponent(String name) throws ManagementException;

    void unregisterInstaller(String name) throws ManagementException;

    void unregisterServiceAssembly(String saName) throws ManagementException;

    void unregisterSharedLibrary(String slName) throws ManagementException;

}
