/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.management;



/**
 * A Task is an execution unit, that can be executed or undo. 
 * <p>
 * Tasks can be used by a processor that will execute them and reverse execution in case of fault.
 * </p>
 * 
 * @author Olivier Fabre - EBM WebSourcing
 */
public interface Task {

    /**
     * Launch the task with the given context. Task can modify information
     * included in this context during execution.
     * 
     * @param context
     *            the execution context handling input and output information
     * @throws Exception
     *             if an error occurs during Task processing
     */
    void execute(final Context context) throws Exception;

    /**
     * Reverse the task in order to retrieve the environ state before execution
     * of this task.
     * 
     * @param context
     *            the execution context handling input and output information
     * @throws Exception
     *             if an error occurs during Task unprocessing
     */
    void undo(final Context context) throws Exception;
}
