/**
 * Copyright (c) 2009-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.log;

/**
 * @author Christophe Hamerling - EBM WebSourcing
 */
public interface Logger {

    /**
     * if log is enabled, Log as <i>DEBUG</i> : -CALL -
     * 
     */
    void call();

    /**
     * if log is enabled, Log as <i>DEBUG</i> : -CALL -
     * 
     * @param msg
     */
    void call(Object msg);

    /**
     * if log is enabled, Log as <i>DEBUG</i> : -START-
     * 
     */
    void start();

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : -START-msg
     * 
     * @param msg
     */
    void start(Object msg);

    /**
     * if log is enabled, Log as <i>DEBUG</i> : - END -
     * 
     */
    void end();

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : - END -msg
     * 
     * @param msg
     */
    void end(Object msg);

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message
     * 
     * @param msg
     */
    void debug(Object message);

    /**
     * if log is enabled, Log as <i>INFO</i> the message
     * 
     * @param msg
     */
    void info(Object message);

    /**
     * if log is enabled, Log as <i>INFO</i> the message and error
     * 
     * @param msg
     * @param error
     */
    void info(Object message, Throwable error);

    /**
     * if log is enabled, Log as warning the message
     * 
     * @param msg
     */
    void warning(Object message);

    /**
     * if log is enabled, Log as warning the exception
     * 
     * @param msg
     * @param throwable
     */
    void warning(Object message, Throwable throwable);

    /**
     * if log is enabled, Log as <i>ERROR</i> the message
     * 
     * @param msg
     */
    void error(Object message);

    /**
     * if log is enabled, Log as <i>ERROR</i> the exception
     * 
     * @param msg
     * @param throwable
     */
    void error(Object message, Throwable throwable);

    /**
     * 
     * @return
     */
    boolean isDebugEnabled();

    /**
     * 
     * @return
     */
    boolean isInfoEnabled();

    /**
     * 
     * @return
     */
    boolean isWarnEnabled();

    /**
     * 
     * @return
     */
    boolean isErrorEnabled();

    String getName();

    /**
     * Allow to return the logger implementation
     * wrapped by LoggingUtil
     * @return <java.util.logging.Logger>
     */
    java.util.logging.Logger getLogger();
}
