/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.container.thread;

import static org.easymock.EasyMock.expect;
import static org.easymock.classextension.EasyMock.createMock;
import static org.easymock.classextension.EasyMock.replay;
import static org.easymock.classextension.EasyMock.reset;
import static org.easymock.classextension.EasyMock.verify;
import static org.junit.Assert.fail;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;

import org.junit.Before;
import org.junit.Test;

/**
 * Component life cycle thread unit test.
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ComponentLifeCycleThreadTest {

    /**
     * The component life cycle thread to test
     */
    private ComponentLifeCycleThread lifeCycleThread;

    /**
     * The mock life cycle
     */
    private ComponentLifeCycle lifeCycleMock;

    /**
     * The mock component
     */
    private Component componentMock;

    /**
     * The mock component context
     */
    private ComponentContext contextMock;

    /**
	 *
	 *
	 */
    @Test
    public void testDoInit() throws Exception {
        lifeCycleThread.start();
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        reset(contextMock);
        expect(contextMock.getComponentName()).andReturn("contextMock");
        replay(contextMock);
        
        lifeCycleThread.doInit(contextMock);
        verify(componentMock);
    }

    /**
	 *
	 *
	 */
    @Test
    public void testDoStart() throws Exception {
        lifeCycleThread.start();
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doStart();
        verify(componentMock);
    }

    /**
	 *
	 */
    @Test
    public void testDoStop() throws Exception {
        lifeCycleThread.start();

        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doStop();
        verify(componentMock);
    }

    /**
	 *
	 *
	 */
    @Test
    public void testDoShutdown() throws Exception {
        lifeCycleThread.start();

        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);

        lifeCycleThread.doShutdown();
        verify(componentMock);
    }

    /**
     * Test multiple calls
     * 
     */
    @Test
    public void testMultipleCalls() {
        lifeCycleThread.start();

        // stop component
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);
        try {
            lifeCycleThread.doStop();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(componentMock);

        // shutdown component
        reset(componentMock);
        expect(componentMock.getLifeCycle()).andReturn(lifeCycleMock);
        replay(componentMock);
        try {
            lifeCycleThread.doShutdown();
        } catch (JBIException e) {
            fail(e.getMessage());
        }
        verify(componentMock);

    }

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    @Before
    public void setUp() throws Exception {

        componentMock = createMock(Component.class);
        contextMock = createMock(ComponentContext.class);
        
        lifeCycleMock = createMock(ComponentLifeCycle.class);

        lifeCycleThread = new ComponentLifeCycleThread(componentMock, 0, null);
    }

}
